##Raster analysis=group
##Change Statistics=name
##Map_date1=raster
##Map_date2=raster
##date1=number 
##date2=number 
##Stat=Output table
##ChangeMatrix=Output table

require(raster)

# Read map resolution
res <- res(Map_date1)
# Area of categories in hectares
freq1 <- as.data.frame(freq(Map_date1))
freq2 <- as.data.frame(freq(Map_date2))
names(freq1) <- names(freq2) <- c("value","count")

# Merge the two tables
tab <- merge(freq1,freq2,by='value',all=T)
tab[is.na.data.frame(tab)]<-0
categories <- tab[,1]
# calculate area in hectares
areasd1 <- round(tab[,2]*res[1]*res[2]/10000)
areasd2 <- round(tab[,3]*res[1]*res[2]/10000)

# Absolute change (ha)
AC <- areasd2 - areasd1
# Relative change (normalized by initial class area)
RC <- 100* (areasd2 - areasd1) / areasd1

# Change rate (similar to deforestation rate used by FAO)
# FAO q = (A2/A1)^(1/(t2-t1))-1
t <- 100*(areasd2/areasd1)^(1/(date2-date1))-100
# Other 1/(t2-t1)xln(A2/A1) derived from the Compound Interest Law, see Puyravaud 2002
r <- 100/(date2-date1)*log(areasd2/areasd1)
# Join everything into a single table
Change_tab <- rbind(areasd1,areasd2,AC,RC,t,r)
rownames(Change_tab) <- c("Area date 1","Area date 2","Absolute change (ha)", "Relative change (%)", "Annual rate of change t (FAO)", "Annual rate of change r")
colnames(Change_tab) <- categories
matrix <- crosstab(Map_date1, Map_date2, digits=0, long=FALSE, useNA=FALSE, progress='')
matrix <- matrix * (res[1]*res[2]/10000)
ChangeMatrix <- matrix

> print("Change indices (areas in ha)")
> print(Change_tab)
> print("Change matrix (areas in ha)")
> print(ChangeMatrix)

Stat <- Change_tab
