##Geographically weighted methods=group
##Layer=vector
##Accuracy=selection Overall;User;Producer
##Reference_data=Field Layer
##Classified_Data=Field Layer
##Category=optional string
##Bandwidth=number 0.15
##Output=output vector

library(sp)
library(spgwr)

spatlayer=as(Layer, "Spatial")
coords=coordinates(spatlayer)
alogit=function(x){exp(x)/(1+exp(x))}

Layer[[Reference_data]]=as.integer(Layer[[Reference_data]])
Layer[[Classified_Data]]=as.integer(Layer[[Classified_Data]])
tab=table(Layer[[Reference_data]], Layer[[Classified_Data]])

if(Accuracy==0){
  spdf=as.data.frame(spatlayer)
  spdf[,"agreement"]=0
  spdf[,"agreement"][Layer[[Reference_data]] == Layer[[Classified_Data]]]=1
  gwr.oa=ggwr(spdf$agreement~1, data=spatlayer, fit.points=coords, adapt=Bandwidth, family=binomial)
  res.oa=alogit(data.frame(gwr.oa$SDF$X.Intercept.))
  res.oa=cbind(gwr.oa$SDF, res.oa)
  spdf=SpatialPointsDataFrame(res.oa, as.data.frame(res.oa))
  Output=st_as_sf(spdf)
  print(summary(res.oa))
  OA=sum(diag(tab))/nrow(Layer)
  print("Global Overall Accuracy:")
  print(OA)
}

if(Accuracy==1){
  lu_codes <- unique(Layer[[Reference_data]])
  df <- as.data.frame(spatlayer)
  for (i in 1:length(lu_codes)){
    df[paste(lu_codes[i], "_REF", sep = "")] <- 0
    df[paste(lu_codes[i], "_CLASS", sep = "")] <- 0
    df[,paste(lu_codes[i], "_REF", sep = "")][Layer[[Reference_data]] == lu_codes[i]] <- 1
    df[,paste(lu_codes[i], "_CLASS", sep = "")][Layer[[Classified_Data]] == lu_codes[i]] <- 1
  }
  y=paste(Category, "_REF", sep = "")
  x=paste(Category, "_CLASS", sep = "")
  gwr.ua=ggwr(df[,y]~df[,x], data=spatlayer, fit.points=coords, adapt=Bandwidth, family=binomial)
  coefs_ua <- rowSums(data.frame(gwr.ua$SDF)[,2:3])
  res.ua=alogit(data.frame(coefs_ua))
  res.ua=cbind(gwr.ua$SDF, res.ua)
  spdf=SpatialPointsDataFrame(res.ua, as.data.frame(res.ua))
  Output=st_as_sf(spdf)
  print(summary(res.ua))
  OA=tab[Category, Category]/sum(tab[Category,])
  print("Global user accuracy:")
  print(OA)
  
}

if(Accuracy==2){
  lu_codes <- unique(Layer[[Reference_data]])
  df <- as.data.frame(spatlayer)
  for (i in 1:length(lu_codes)){
    df[paste(lu_codes[i], "_REF", sep = "")] <- 0
    df[paste(lu_codes[i], "_CLASS", sep = "")] <- 0
    df[,paste(lu_codes[i], "_REF", sep = "")][Layer[[Reference_data]] == lu_codes[i]] <- 1
    df[,paste(lu_codes[i], "_CLASS", sep = "")][Layer[[Classified_Data]] == lu_codes[i]] <- 1
  }
  y=paste(Category, "_CLASS", sep = "")
  x=paste(Category, "_REF", sep = "")
  gwr.pa=ggwr(df[,y]~df[,x], data=spatlayer, fit.points=coords, adapt=Bandwidth, family=binomial)
  coefs_pa <- rowSums(data.frame(gwr.pa$SDF)[,2:3])
  res.pa=alogit(data.frame(coefs_pa))
  res.pa=cbind(gwr.pa$SDF, res.pa)
  spdf=SpatialPointsDataFrame(res.pa, as.data.frame(res.pa))
  Output=st_as_sf(spdf)
  print(summary(res.pa))
  PA=tab[Category, Category]/sum(tab[,Category])
  print("Global producer accuracy:")
  print(PA)
}

